#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Core {
  public class OperatorCallNode : CallNodeBase {
    [Storable]
    private IOperator myOperator;
    public IOperator Operator {
      get { return myOperator; }
      set {
        if (myOperator != value) {
          ClearNameMappings();
          myOperator = value;
          OnOperatorChanged();
          foreach (IParameter parameter in value.Parameters)
            AddNameMapping(parameter.Name, parameter.Name);
        }
      }
    }

    public override ICloneable Clone(ICloner cloner) {
      OperatorCallNode clone = (OperatorCallNode)base.Clone(cloner);
      clone.Operator = (IOperator)cloner.Clone(Operator);
      return clone;
    }

    public event EventHandler OperatorChanged;
    protected virtual void OnOperatorChanged() {
      if (OperatorChanged != null)
        OperatorChanged(this, new EventArgs());
    }
  }
}
