#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Core {
  public class Parameter : ItemBase, IParameter {
    [Storable]
    private string myName;
    public string Name {
      get { return myName; }
    }

    [Storable]
    private string myDescription;
    public string Description {
      get { return myDescription; }
    }

    [Storable]
    private Type myDataType;
    public Type DataType {
      get { return myDataType; }
    }

    [Storable]
    private ParamterType myType;
    public ParamterType Type {
      get { return myType; }
    }

    public Parameter() {
      myName = "Anonymous";
      myDescription = "";
      myDataType = null;
      myType = ParamterType.In;
    }
    public Parameter(string name, string description, Type dataType, ParamterType type)
      : this() {
      myName = name;
      myDescription = description;
      myDataType = dataType;
      myType = type;
    }

    public override IView CreateView() {
      return new ParameterView(this);
    }

    public override ICloneable Clone(ICloner cloner) {
      Parameter clone = new Parameter();
      cloner.AddClonedObject(this, clone);
      clone.myName = Name;
      clone.myDescription = Description;
      clone.myDataType = DataType;
      clone.myType = Type;
      return clone;
    }
  }
}
